/*
    MegaMania-pc, (c) Paolo Lucchesi, 2001
    Game routines
*/

#include <stdio.h>
#include <stdlib.h>
#include <allegro.h>
#include "data.h"
#include "const.h"
#include "newtypes.h"

extern DATAFILE *data;
extern char options[NUMOPTS];
extern int nosound;
extern int worstnightmare;

extern int logging;
extern FILE* log;

// Variabile e funzione per il timer
volatile int scnt;
void ticktimer() { scnt++; }
END_OF_FUNCTION(ticktimer);

void* shipdrawn;

fixed px,dx;
shot myshot;
shot enshot[MAXENSHOT];
int numshots;
enemy foes[MAXENEMY];
int numenemy;
int enspr,endis;

int shipleft;
long score;
int round;
int totround;
int megacycle;
int energy,enticks;
int paused;

int infoupdate;

//FILE* log;

long game(int worst) {
    // Imposta le bitmap, setta il timer e chiama gamesequence
    //log=fopen("mm.log","wt"); fprintf(log,"Logging...\n");
    create_bitmaps();
    LOCK_VARIABLE(scnt);
    LOCK_FUNCTION(ticktimer);
    if (options[3] || worstnightmare)
        install_int_ex(ticktimer,BPS_TO_TIMER(HIGHSPEED));
     else
        install_int_ex(ticktimer,BPS_TO_TIMER(LOWSPEED));
    scnt=0;

    gamesequence();
    if (paused<0) return(score);
    if (!nosound) play_sample(data[MDAT_SAMPGAMEOVER].dat,255,128,1000,0);
    rest(2000);
    if (score>=MAXSCORE) {
        if (!nosound) play_sample(data[MDAT_SAMPGAMEOVER].dat,255,128,1200,0);
        rest(1800);
        if (!nosound) play_sample(data[MDAT_SAMPGAMEOVER].dat,255,128,1400,0);
        rest(1600);
        if (!nosound) play_sample(data[MDAT_SAMPGAMEOVER].dat,255,128,1600,0);
        rest(1300);
    }
    return(score);
}

int gamesequence() {
    // Sequenza principale della partita
    int newround=1,i;
        // newround indica se sto iniziando un nuovo round o se sto
        // ripartendo su un round iniziato con una vita precedente
    gameinit(); // Inizializzo la partita
    if (worstnightmare) round=rand()%8;
    for (;;) {
        foesinit(round,newround); //lvl
#ifndef RELEASE
        if (logging) {
            fprintf(log,"\nRound %d, New:%d, Cycle %d\n",round,newround,megacycle);
            for (i=0;i<MAXENEMY;i++)
                fprintf(log,"%2d:%3d (%4d,%4d) |",i,foes[i].st,foes[i].x,foes[i].y);
            fprintf(log,"\n");
        }
#endif
        // Quanti colpi possono sparare i nemici?
        numshots=1;
        if (!(round%2)) numshots++;
        if (worstnightmare) numshots++;
        if (megacycle) numshots++;
        if (numshots>MAXENSHOT) numshots=MAXENSHOT;
        fill_energy();
        // gameloop gestisce un singolo round
        if (gameloop() > 0) {
            // Round finito per eliminazione di tutti i nemici: punti per
            // energy rimasta, passo al prossimo round
            score_energy();
            if (score>=MAXSCORE) break; //wow! 999999
            totround++;
            if (worstnightmare) {
                round=rand()%8;
                if (!(totround%4)) megacycle++;
            } else {
                round++;
                if (round>=NUMROUND) {round=0; megacycle++; }
            }
            newround=1;
        } else {
            // Round finito per esplosione della navicella: animazione della
            // botta forte, levo una ship, ricentro, ripeto lo stesso round
            if (paused<0) break;
            ship_explode();
            if (shipleft<=0) break; // GAME OVER
            shipleft--;
            px=SHIPSTARTPOS<<16;
            newround=0;
        }
#ifndef RELEASE
        if (logging) {
            fprintf(log,"Ending:\n");
            for (i=0;i<MAXENEMY;i++)
                fprintf(log,"%2d:%3d (%4d,%4d) |",i,foes[i].st,foes[i].x,foes[i].y);
            fprintf(log,"\n");
        }
#endif
    }
    infoupdate=1; display((score>=MAXSCORE) ? 0 : 7);
    return(0);
}

int gameloop() {
    int res,i;

    // Chiamo glogic temporizzandola, fino a che non ritorna !=0
    for (;;) {
        if (scnt<0) display(0);
        while (scnt<0);
        if (res=glogic()) break;
        scnt--;
    }
    // Blocco tutti gli spari
    myshot.st=0;
    for (i=0;i<4;i++) enshot[i].st=0;
    // Ritorno il risultato dell'ultima gamelogic
    return(res);
}

int gameinit() {
    // Inizializzo la partita
    shipleft=NUMLIVES;
    score=0;
    round=0;
    totround=0;
    megacycle=0;
    paused=0;
    px=SHIPSTARTPOS<<16;
    return 0;
}

int roundinit() {
    // Inizializzo il singolo round
    dx=0; // Ship ferma, no movimento inerziale
    shipdrawn=data[MDAT_SHIPC].dat;
    energy=MAXNRG; enticks=NENTICKS;
    enspr=0;
    return 0;
}

int glogic() {
    // Operazioni da svolgere ad ogni tick
    if (paused) return(do_pause());
    cheats();
    shipmove();
    if (myshot.st) myshot_move();
    foesmove(round,megacycle,0);
    enspr=(enspr+1)%256; // Seleziono il nuovo sprite dei nemici
    if (round==2 || round==6) enspr=(enspr+1)%256;
    enshot_fire();
    enshot_move();
    if (update_energy(-1)<=0) return(-1); // Fine energia?
    return(foestest(round));
}

int update_energy(int delta) {
    // Aggiorna l'energia di delta "ticks". Se enticks esce dall'intervallo
    // [0..NENTICKS[, energy viene aggiornata e impostata per la visualizzazione
    enticks+=delta;
    while (enticks<0) { /// ???? < o <=
        energy--;
        infoupdate=1;
        enticks+=NENTICKS;
    }
    while (enticks>=NENTICKS) {
        energy++;
        infoupdate=1;
        enticks-=NENTICKS;
    }
    return(energy);
}

int fill_energy() {
    // Animazione della barra di energia che si riempie (da 0)
    energy=0; enticks=0;
    infoupdate=1;
    if (!nosound) play_sample(data[MDAT_SAMPENFILL].dat,255,128,450,1);
    for (;;) {
        if (scnt<0) display(0);
        while (scnt<0);
        if (update_energy(20)>=MAXNRG) break;
        scnt--;
    }
    energy=MAXNRG; enticks=NENTICKS;
    stop_sample(data[MDAT_SAMPENFILL].dat);
    return(0);
}

int score_energy() {
    // Animazione della barra di energia residua che si svuota, incrementando
    // il punteggio
    int lasten;

    enticks=NENTICKS;
    lasten=energy;
    if (!nosound) play_sample(data[MDAT_SAMPENSCORE].dat,255,128,1000,1);
    for (;;) {
        if (score>=MAXSCORE) break; //wow! 999999
        if (scnt<0) display(0);
        while (scnt<0);
        update_energy(-20);
        if (energy<lasten) {
            addscore();
            lasten=energy;
        }
        if (energy<=0) break;
        scnt--;
    }
    stop_sample(data[MDAT_SAMPENSCORE].dat);
    return(0);
}

int ship_explode() {
    // Animazione della ship che esplode
    int eph,cpph;

    infoupdate=1;
    dx=0;
    eph=1; cpph=0;
    if (!nosound) play_sample(data[MDAT_SAMPEXPLODE].dat,255,128,1000,0);
    // eph e' il numero di sprite di esplosione, cpph un counter
    for (;;) {
        if (scnt<0) {
            display(eph);
        }
        while (scnt<0);
        cpph++;
        if (cpph>=20 && eph<7) {cpph=0; eph++; }
        if (eph>=7 && cpph>=200) break;
        scnt--;
    }
    return(0);
}

int shipmove() {
    // Movimento dell'astronave e input del giocatore
    if (options[2]) {
        // Movimento inerziale:
        // In funzione dei tasti <- e -> premuti aggiorno l'inerzia
        if (key[KEY_LEFT] && dx>(-1<<16)) dx-=INERTIAINC;
        if (key[KEY_RIGHT] && dx<(1<<16)) dx+=INERTIADEC;
        // Se nessun tasto e' premuto, diminuisco l'inerzia
        if (!key[KEY_LEFT] && !key[KEY_RIGHT]) {
            if (dx<0) dx+=INERTIADEC;
            if (dx>0) dx-=INERTIADEC;
        }
    } else {
        // Movimento non inerziale
        dx=0;
        if (key[KEY_LEFT] && !key[KEY_RIGHT]) dx=(-1<<16);
        if (key[KEY_RIGHT] && !key[KEY_LEFT]) dx=(1<<16);
    }
    // Fire?
    if ((key[KEY_LCONTROL] || key[KEY_UP]) && !myshot.st) myshot_fire();
    px+=dx; // Aggiorno posizione in base all'inerzia
    // Limito movimento ai bordi
    if (key[KEY_P]) paused=1;
    if (px<(CX1<<16)) px=(CX1<<16);
    if (px>((CX2-BRD)<<16)) px=((CX2-BRD)<<16);
    return(0);
}

int do_pause() {
    int i,k;

    k=0;
    for (i=1;i<128;i++) k= (k || key[i]);
    if (!k && paused==1) paused=2;
    if (k && paused==2) paused=0;
    if (key[KEY_ESC]) {paused=-1; return(paused); }
    return(0);
}

int cheats() {
#ifndef RELEASE
    // Debugging cheats
    if (key[KEY_E] && key[KEY_RSHIFT]) shipleft=MAXLIVES;   // Bonus lives
    if (key[KEY_L] && key[KEY_RSHIFT]) numenemy=0;          // End round
    if (key[KEY_S] && key[KEY_RSHIFT]) score=999900;        // Big Points
    if (key[KEY_D] && key[KEY_RSHIFT]) energy=0;            // Die
    if (key[KEY_F] && key[KEY_RSHIFT]) energy=80;           // Energyfill
    if (key[KEY_PRTSCR]) makescreenshot();                  // Screenshot
    if (key[KEY_B]) makescreenshot();                  // Screenshot
#endif
    // Real cheats (per ora nessuno)
    return(0);
}

int myshot_fire() {
    // Procedura di fuoco "amico"
    if (myshot.st) return -1;
    myshot.st=1;
    myshot.x=(px>>16)+14;
    myshot.y=SHY;
    if (!nosound) play_sample(data[MDAT_SAMPSHOT].dat,255,128,1000,0);
}

int myshot_move() {
    // Movimento del fuoco amico
    myshot.y-=2;
    if (options[1]) myshot.y--;
    if (myshot.y <= 0) myshot.st=0;
    if (options[0]) myshot.x=(px>>16)+14;
    return(myshot.st);
}

int enshot_fire() {
    // Procedura di fuoco "nemico"
    int kx,ky,kn,i;

    if (rand()%ENEMYFIRERATE) return(0);
    // Colpi a disposizione?
    kn=rand()%numshots;
    if (enshot[kn].st) return(0);
    //if (kn>=numshots) return(0);

    // Cerco il nemico piu' basso (o uno dei piu' bassi)
    ky=-1;
    for (i=0;i<MAXENEMY;i++) {
        if (foes[i].st==1) {
            if (foes[i].y > ky || (foes[i].y==ky && !(rand()%4))) {
                kx=foes[i].x;
                ky=foes[i].y;
            }
        }
    }
    // Se l'ho trovato
    if (ky>=0) {
        enshot[kn].st = 1;
        enshot[kn].x = kx+14;
        enshot[kn].y = ky+12;
        return(1);
    }
    return(0);
}

int enshot_move() {
    // Movimento di fuoco nemico
    int i;
    for (i=0;i<MAXENSHOT;i++) {
        if (enshot[i].st) {
            enshot[i].y+=2;
            if (enshot[i].y >= (SHY+48)) enshot[i].st=0;
        }
    }
    return(0);
}

int foestest(int level) {
    // Verifiche varie sui nemici
    int i;
    // Controllo la collisione tra i nemici e il proiettile "amico"
    if (myshot.st) {
      for (i=0;i<MAXENEMY;i++) {
        if (foes[i].st>0) {
          if (spritecollide(data[MDAT_ENEMY00+endis].dat,foes[i].x,foes[i].y,
                            data[MDAT_SHOTMY].dat,myshot.x,myshot.y)) {
                foes[i].st=-32;
                myshot.st=0;
                addscore();
                if (!nosound) play_sample(data[MDAT_SAMPFOEEXPL].dat,255,128,1000,0);
                break;
          }
        }
      }
    }
    // Controllo la collisione tra i nemici e la ship
    for (i=0;i<MAXENEMY;i++) {
      if (foes[i].st>0) {
        if (spritecollide(data[MDAT_ENEMY00+endis].dat,foes[i].x,foes[i].y,
                          shipdrawn,fixtoi(px),SHY)) {
                return(-1);
        }
      }
    }
    // Se un nemico e' esplodente, aggiorno il suo status
    for (i=0;i<MAXENEMY;i++) {
        if (foes[i].st<0) {
            foes[i].st++;
            if (foes[i].st==0) numenemy--;
        }
    }
    // Controllo la collisione tra proiettili nemici e ship
    for (i=0;i<MAXENSHOT;i++) {
        if (enshot[i].st) {
            if (spritecollide(data[MDAT_SHOTEN].dat,enshot[i].x,enshot[i].y,
                              shipdrawn,fixtoi(px),SHY)) {
                 return(-1);
            }
        }
    }
    if (numenemy<=0) return(1);         // Fine Round?
    if (score>=MAXSCORE) return(1);     // wow! 999999
    return(0);
}

int addscore() {
    // Incrementa il punteggio
    int t;

    t=score/BONUSSCORE;
    if (totround>7 && !worstnightmare)  score+=90;
     else                               score+=20+10*totround;
    if (score>MAXSCORE) score=MAXSCORE;
    if ((score/BONUSSCORE)>t && shipleft<MAXLIVES && !worstnightmare) shipleft++;
    infoupdate=1;
}

int spritecollide(BITMAP* spr1, int x1, int y1, BITMAP* spr2, int x2, int y2) {
    /* Routine di riconoscimento collisione sprite
       spr1 e spr2 sono puntatori a due sprite in posizione (x1,y1) e
       (y1,y2). Ritorna 1 se collidono, 0 altrimenti
    */
    int sx1=0,sy1=0,sx2=0,sy2=0;
    int dx,dy;
    register int i,j;

    // Controllo se le box che contengono gli sprite si intersecano
    if (x2>=x1+spr1->w) return(0);
    if (y2>=y1+spr1->h) return(0);
    if (x1>=x2+spr2->w) return(0);
    if (y1>=y2+spr2->h) return(0);

    // Determino la box di intersezione
    if (x1<x2) sx1=x2-x1; else sx2=x1-x2;
    if ((spr1->w - sx1)<(spr2->w - sx2)) dx=spr1->w - sx1; else dx=spr2->w - sx2;
    if (y1<y2) sy1=y2-y1; else sy2=y1-y2;
    if ((spr1->h - sy1)<(spr2->h - sy2)) dy=spr1->h - sy1; else dy=spr2->h - sy2;

    // Eseguo il controllo pixelxpixel nella box di intersezione
    for (i=0; i<dx; i++)
        for (j=0; j<dy; j++) {
            if (spr1->line[sy1+j][sx1+i] && spr2->line[sy2+j][sx2+i]) return(1);
    }
    return(0);
}

#ifndef RELEASE

int makescreenshot() {
    // Crea uno screenshot in formato .bmp
    FILE *bmp;
    int i,j,r;
    RGB col;

    if ((bmp=fopen("scrshot.bmp","wb"))==NULL) return(-1);
    makebmpheader(XSIZE,YSIZE,bmp);
    for (i=0;i<256;i++) {
        get_color(i,&col);
        putc(4*col.b,bmp); putc(4*col.g,bmp); putc(4*col.r,bmp);
        putc(0,bmp);
    }
    for (i=YSIZE-1;i>=0;i--) {
        for (j=0;j<XSIZE;j++) {
            r=getpixel(screen,j,i);
            putc((unsigned char)r,bmp);
        }
    }
    return(0);
}

inline int putlong(long v, FILE *h) { return(fwrite(&v,sizeof(long),1,h)); }
inline int putshort(short v, FILE *h) { return(fwrite(&v,sizeof(short),1,h)); }

int makebmpheader(long x, long y, FILE* out) {
    long lg;
    short sh;

    putc('B',out); putc('M',out);
    putlong(x*y+1078,out);
    putlong(0,out);
    putlong(1078,out);
    putlong(40,out);
    putlong(x,out);
    putlong(y,out);
    putshort(1,out);
    putshort(8,out);
    putlong(0,out);
    putlong(0,out);
    putlong(3396,out);
    putlong(3396,out);
    putlong(0,out);
    putlong(0,out);
}

#endif


