/*
    MegaMania-pc, (c) Paolo Lucchesi, 2001
    Graphic routines
*/

#include <stdlib.h>
#include <allegro.h>
#include "data.h"
#include "const.h"
#include "newtypes.h"

extern DATAFILE *data;

// Bitmap del playfield e della parte info, usate per il dblbuffer
BITMAP* pfbmp;
BITMAP* isbmp;

struct {
    int x;
    int y;
    int c;
} star[NUMSTARS];

extern fixed px,dx;
extern shot myshot;
extern shot enshot[MAXENSHOT];
extern enemy foes[MAXENEMY];
extern int enspr,endis;

extern int shipleft;
extern int score;
extern int round;
extern int megacycle;
extern int energy,enticks;
extern int paused;

extern int infoupdate;
extern void* shipdrawn;

void create_stars() {
    int i;

    for (i=0;i<NUMSTARS;i++) {
        star[i].x = CX1+rand()%(CX2-CX1);
        star[i].y = CY1+rand()%(CY2-CY1);
        star[i].c = 240+rand()%3;
    }
}

void update_stars() {
    int i;

    for (i=0;i<NUMSTARS;i++) {
        star[i].y+=2;
        if (star[i].y >= CY2) {
            star[i].y = CY1;
            star[i].x = CX1+rand()%(CX2-CX1);
        }
    }
}

void draw_stars() {
    int i;

    for (i=0;i<NUMSTARS;i++) putpixel(pfbmp,star[i].x,star[i].y,star[i].c);
}

void create_bitmaps() {
    pfbmp=create_bitmap(640,400); clear(pfbmp);
    isbmp=create_bitmap(640,80); clear(isbmp);
    set_clip(pfbmp,CX1,CY1,CX2-1,CY2-1);
    create_stars();
}

void destroy_bitmaps() {
    destroy_bitmap(isbmp);
    destroy_bitmap(pfbmp);
}

void display_info() {
    // Disegna tutta la parte inferiore dello schermo, quella che
    // contiene punteggio, energia e vite rimaste
    int i,en;

    clear_to_color(isbmp,240);
    for (i=0;i<shipleft;i++)
        draw_sprite(isbmp,data[MDAT_HSHIP].dat,306+i*48,36);
    textprintf(isbmp,data[MDAT_FONT].dat,66,22,0,"%6d",score);
    textprintf(isbmp,data[MDAT_FONT].dat,64,20,241,"%6d",score);
    en=3*energy;
    rect(isbmp,361,16,602,26,0);
    rect(isbmp,360,15,601,25,242);
    rectfill(isbmp,361,16,361+en,24,241);
    if (en<240) rectfill(isbmp,362+en,16,600,24,0);
    textout(isbmp,font,"Energy",302,18,0);
    textout(isbmp,font,"Energy",300,16,242);
}

void foesdisplay(int level) {
    // Disegna i nemici
    int i,kdis;

    // Calcolo l'indice dello sprite da disegnare
    endis=level*8+(enspr/64);
    for (i=0;i<MAXENEMY;i++) {
        if (foes[i].st==1) {                    // Nemico vivo
            draw_sprite(pfbmp,data[MDAT_ENEMY00+endis].dat,foes[i].x,foes[i].y);
        }
        if (foes[i].st<0) {                     // Nemico esplodente
            kdis=level*8+4+((32+foes[i].st)/8);
            draw_sprite(pfbmp,data[MDAT_ENEMY00+kdis].dat,foes[i].x,foes[i].y);
        }
    }
}

void display(int tp) {
    // Routine grafica, disegna il tutto
    int vs,i;

    // Scelgo lo sprite in funzione dell'inerzia
    vs=0;
    if (dx<(-3<<14)) vs--;
    if (dx<(-1<<14)) vs--;
    if (dx>(1<<14)) vs++;
    if (dx>(3<<14)) vs++;

    // Preparo la playfield bitmap
    // Questa parte probabilmente puo' essere ottimizzata "a bestia"
    clear(pfbmp);
    if (!paused) update_stars();
    draw_stars();
    // Disegno i vari proiettili
    if (myshot.st) {
        draw_sprite(pfbmp,data[MDAT_SHOTMY].dat,myshot.x,myshot.y);
    }
    for (i=0;i<MAXENSHOT;i++) if (enshot[i].st) {
        draw_sprite(pfbmp,data[MDAT_SHOTEN].dat,enshot[i].x,enshot[i].y);
    }
    // Scelgo lo sprite della ship e lo disegno
    if (tp) {
        shipdrawn=data[MDAT_SHIPE0+tp].dat;
    } else {
        switch(vs) {
            case -2: shipdrawn=data[MDAT_SHIPL].dat; break;
            case -1: shipdrawn=data[MDAT_SHIPML].dat; break;
            case 1:  shipdrawn=data[MDAT_SHIPMR].dat; break;
            case 2:  shipdrawn=data[MDAT_SHIPR].dat; break;
            default: shipdrawn=data[MDAT_SHIPC].dat;
        }
    }
    draw_sprite(pfbmp,shipdrawn,fixtoi(px),SHY);
    foesdisplay(round);
    if (infoupdate) display_info();
    // Double buffer here
    vsync();
    blit(pfbmp, screen, 0, 0, 0, 0, 640, 400);
    if (infoupdate) blit(isbmp, screen, 0, 0, 0, 400, 640, 80);
    infoupdate=0;
}






