/*
    MegaMania-pc, (c) Paolo Lucchesi, 2001
    Main routines
*/

#ifndef WIN
    #define USE_CONSOLE 1
#endif
#include <stdio.h>
#include <stdlib.h>
#include <allegro.h>
#include "data.h"
#include "const.h"

DATAFILE *data;

long lastscore;
long stdhigh;
long worsthigh;
char options[NUMOPTS];

int nosound; // nosound=2 -> NO SOUND, nosound=1 -> init error
int logging;
int grmode;
FILE* log;
int worstnightmare;

int main(int argc, char** argv) {
    int r,card;

    nosound=0; logging=0; grmode=0;
    testswitches(argc,argv);
    // Inizializzazioni varie di Allegro
    allegro_init();
    install_keyboard();
    install_timer();
    card=GFX_AUTODETECT;
#ifdef WIN
    if (grmode>0) card=GFX_AUTODETECT_FULLSCREEN;
    if (grmode<0) card=GFX_AUTODETECT_WINDOWED;
#endif
    set_gfx_mode(card,XSIZE,YSIZE,0,0);
#ifdef WIN
    set_window_title(VERSION);
#endif
    text_mode(-1);
    // Disabilita CTRL-ALT-END in release ufficiale
#ifdef RELEASE
    three_finger_flag=0;
#endif
    data=load_datafile("megaman.dat");
#ifndef RELEASE
    if (logging)    if ((log=fopen("megaman.log","wt"))==NULL) logging=0;
#endif
    set_palette(data[MDAT_PALETTE].dat);
    // Setto colori
    set_color_t(240,32,16,8);
    set_color_t(241,32,48,48);
    set_color_t(242,16,48,24);

    load_ini();
    lastscore=0;
    beginsound();
    startmidi();
    for (;;) {
        clear(screen);
        r=intro();
        if (r==3) break;                        // Quit game
        if (r==2) { optionpage(); continue; }   // Options
        killmidi();
        worstnightmare=r;
        lastscore=game(); // E vai...
        if (r==0 && lastscore>stdhigh) stdhigh=lastscore;
        if (r==1 && lastscore>worsthigh) worsthigh=lastscore;
        startmidi();
    }
#ifdef RELEASE
    save_ini();
#endif
    /// Bye bye, allegro... Bye bye, everybody
    allegro_exit();
    return(0);
}

int beginsound() {
    // Accrocco per far funzionare il suono anche la prima volta...
    install_sound(DIGI_AUTODETECT,0,NULL);
    return(0);
}

int startmidi() {
    // Inizializza il suono su "solo midi" e fa partire la sigla
    if (nosound>=2) return(0);
    remove_sound();
    if (install_sound(0,MIDI_AUTODETECT,NULL)) return(-1);
    play_looped_midi(data[MDAT_SONG].dat,33,113);
    return(0);
}

int killmidi() {
    // Stoppa la sigla e inizializza il suono su "solo digi"
    if (nosound>=2) return(0);
    stop_midi(); rest(200);
    remove_sound();
    if (install_sound(DIGI_AUTODETECT,0,NULL)) {
        nosound=1;
        return(-1);
    }
    return(0);
}

int intro() {
    // Crea la schermata principale di introduzione, poi gestisce le scelte
    // e ritorna il numero della scelta fatta
    char *choiches[4]={ "Standard Nightmare","Your Worst Nightmare",
                        "Options", "Quit Game" };
    int ci=0, kk;
    char strapp[7];

    clear(screen);
    draw_sprite(screen,data[MDAT_TITLE].dat,0,0);
    textout(screen,font,"Last Score:",448,68,241);
    sprintf(strapp,"%6d",lastscore);
    outstring(strapp,448,80,242,240);
    textout(screen,font,"High Score:",448,132,241);
    sprintf(strapp,"%6d",stdhigh);
    outstring(strapp,448,144,242,240);
    textout(screen,font,"Worst Hiscore:",448,196,241);
    sprintf(strapp,"%6d",worsthigh);
    outstring(strapp,448,208,242,240);

    textout(screen,font,VERSION,32,460,240);
#ifndef RELEASE
    textout(screen,font,"Test Build",460,460,240);
    triangle(screen,18,348,34,340,34,356,240);
    triangle(screen,16,346,32,338,32,354,242);
    triangle(screen,624,348,608,340,608,356,240);
    triangle(screen,622,346,606,338,606,354,242);
#endif
    // Wait key... ESC => esce
    for (;;) {
        rectfill(screen,40,320,599,370,0);
        textout_centre(screen,data[MDAT_FONT].dat,choiches[ci],322,322,240);
        textout_centre(screen,data[MDAT_FONT].dat,choiches[ci],320,320,242);
        waitkey(0); waitkey(1);
        kk=0;
        for (;;) {
            if (key[KEY_ESC]) { ci=3; kk=1; break;}
            if (key[KEY_LCONTROL] || key[KEY_RCONTROL] ||
                key[KEY_SPACE] || key[KEY_ENTER])       { kk=1; break; }
            if (key[KEY_LEFT])  { ci = (ci>0) ? ci-1: 3; break; }
            if (key[KEY_RIGHT]) { ci = (ci<3) ? ci+1: 0; break; }
        }
        if (kk) break;
    }
    return(ci);
}

int optionpage() {
    // Stampa la schermata del menu delle opzioni, gestisce il menu
    // e aggiorna le opzioni di conseguenza
    char *optxt[NUMOPTS+1] = { "Guided missiles", "Fast missiles",
                       "Inertial movement", "High game speed",
                       "Back to main screen" };
    char *yesno[2] = {"no","yes"};
    int i,k;
    int p=0;
    for (;;) {
        vsync();
        clear(screen);
        for (i=0;i<NUMOPTS;i++) options[i] = options[i] ? 1 : 0;
        outstring("Options:",96,32,241,240);
        for (i=0;i<=NUMOPTS;i++) {
            k = (p==i) ? 241 : 242;
            outstring(optxt[i],64,96+48*i,k,240);
            if (i<NUMOPTS) outstring(yesno[options[i]],512,96+48*i,k,240);
        }
        waitkey(0); waitkey(1);
        if (key[KEY_ESC]) break;
        if (key[KEY_LCONTROL] || key[KEY_RCONTROL] ||
            key[KEY_SPACE] || key[KEY_ENTER])           {
                if (p>=NUMOPTS) break;
                options[p]=1-options[p];
        }
        if (key[KEY_LEFT] || key[KEY_UP])       p = (p>0) ? p-1: NUMOPTS;
        if (key[KEY_RIGHT] || key[KEY_DOWN])    p = (p<NUMOPTS) ? p+1: 0;
    }
    return(0);
}

int waitkey(int mode) {
    // Se mode=0, aspetta che nessun tasto sia premuto
    // se mode!=0, aspetta che almeno un tasto sia premuto
    int i,k;
    for (;;) {
        k=0;
        for (i=1;i<128;i++) k= (k || key[i]);
        if ((k && mode) || (!k && !mode)) break;
    }
    return(k);
}

int outstring(char* s, int x, int y, int mc, int sc) {
    // printa la stringa s alle coord(x,y) con colore mc e ombra sc
    textout(screen,data[MDAT_FONT].dat,s,x+2,y+2,sc);
    textout(screen,data[MDAT_FONT].dat,s,x,y,mc);
    return(0);
}

int set_color_t(int c, int r, int g, int b) {
    // Setta il colore c alla tripletta (r,g,b). Ritorna c.
    RGB col;
    col.r=r; col.g=g; col.b=b;
    set_color(c,&col);
    return(c);
}

int load_ini() {
    // Carica il file megaman.ini... in caso di errore inizializza i dati
    FILE *h;

    if ((h=fopen("megaman.ini","rb"))==NULL) {
        initall();
        return(1);
    }
    fread(&stdhigh,sizeof(long),1,h);
    fread(&worsthigh,sizeof(long),1,h);
    fread(options,sizeof(char),NUMOPTS,h);
    return(0);
}

int save_ini() {
    // Salva il file megaman.ini
    FILE *h;

    if ((h=fopen("megaman.ini","wb"))==NULL) return(-1);
    fwrite(&stdhigh,sizeof(long),1,h);
    fwrite(&worsthigh,sizeof(long),1,h);
    fwrite(options,sizeof(char),NUMOPTS,h);
    return(0);
}

int initall() {
    // Inizializza hiscores e opzioni
    int i;
    stdhigh=0;
    worsthigh=0;
    for (i=0;i<NUMOPTS;i++) options[i]= (i<=2) ? 1 : 0;
}

#define NSWT 15
int testswitches(int argc, char** argv) {
    char *swt[NSWT]={"nosound", "--nosound", "-n",
                     "mute", "--mute", "-m",
                     "log", "--log", "-l",
                     "fullscreen", "--fullscreen", "-f",
                     "windowed", "--windowed", "-w"};
    int svl[NSWT]={1,1,1,1,1,1,2,2,2,3,3,3,4,4,4};
    int i,j,r;

    for (i=1;i<argc;i++) {
        r=0;
        for (j=0;j<NSWT;j++) if (!stricmp(argv[i],swt[j])) r=svl[j];
        switch(r) {
            case 1: nosound=2; break;
            case 2: logging=1; break;
            case 3: grmode=1; break;
            case 4: grmode=-1; break;
        }
    }
    return 0;
}
