/*
    MegaMania-pc, (c) Paolo Lucchesi, 2001
    Enemy initialization e movement routines
*/

#include <stdlib.h>
#include <allegro.h>
#include "data.h"
#include "const.h"
#include "newtypes.h"

extern enemy foes[MAXENEMY];
extern int numenemy;

int foesinit(int level, int newround) {
    // Inizializza la posizione dei nemici
    // Level e' il n. di livello, newround indica se deve resettare il livello
    int i,j,n,bx;

    // Se newround azzera tutti i nemici e in ogni caso azzera i nemici morti
    for (i=0;i<MAXENEMY;i++) {
        if (newround || foes[i].st<0) foes[i].st=0;
        if (foes[i].st>0) foes[i].st=2;
    }
    if (level%2) {
        // Livelli 1,3,5 e 7 : disposizione su sei file di tre
        for (i=0;i<6;i++) {
            bx=240;
            // Solo al livello 5 i nemici sono perfettamente incolonnati
            if (level!=5 && i>0) bx=128+rand()%224;
            for (j=0;j<3;j++) {
                n=3*i+j;
                foes[n].dx=1;
                if (level==3 && i%2) foes[n].dx=-1;
                foes[n].spr=0;
                foes[n].x=bx+128*j;
                foes[n].y=-(i+1)*64;
                if (newround) foes[n].st=2;
            }
        }
    } else {
        // Livelli 0,2,4 e 6 : disposizione su 3 file a scacchiera
        for (i=0;i<15;i++) {
            foes[i].dx=1;
            foes[i].spr=0;
            foes[i].x=-640-64+64*((i*2+1)/3);
            foes[i].y=48+48*((i+1)%3);
            if (newround) foes[i].st=2;
        }
    }
    foesmove(level,0,1); // Inizializza le variabili statiche di foesmove
    // Conta i nemici
    numenemy=0;
    for (i=0;i<MAXENEMY;i++) if (foes[i].st) numenemy++;
    return(0);
}

int foesmove(int level, int diff, int init) {
    // Sposta i nemici secondo un determinato schema di movimento
    // Level e' il n. di livello, diff indica che devo usare uno schema di
    // movimento avanzato, init indica che devo solo resettare le var statiche
    static int st;
    static int vs,kmy,dx,dy;
    int i,ksd;

    if (init) { st=0; vs=1; kmy=0; dx=0; dy=0; return(0); }
                                            // Inizializza le var.statiche
    switch(level) {
        case 1: case 3:
            dy=0;
            if (!diff) {
                ksd=st/64;
                if (st==0) vs=-vs;
                if (ksd==0 || ksd==2) dy=1;
                dx=vs;
            } else {
                dy=1;
                if (level==1)   dx=(st&256) ? 1 : -1;
                 else           dx=(st&64) ? 1 : -1;
            }
        break;
        case 5:
            if (st&256) {
                switch(rand()%5) {
                    case 0: dx=-1; dy=0; break;
                    case 1: dx=-1; dy=1; break;
                    case 2: dx=0;  dy=1; break;
                    case 3: dx=1;  dy=1; break;
                    case 4: dx=1;  dy=0; break;
                }
                // Per evitare che i nemici vadano per campi
                for (i=0;i<MAXENEMY;i++) {
                    if (foes[i].st>0) {
                        if (foes[i].x > CX2) dx=-1;
                        if (foes[i].x < (CX1-BRD)) dx=1;
                        break;
                    }
                }
            }
        break;
        case 7:
            dy=1;
            dx=0;
            if (diff) dx=(st&128) ? 1 : -1;
        break;
        case 0: case 2: case 4: case 6:
            if (!diff) {
                dx=1;
            } else {
                dx=1;
                if (st<64) dx=2;
                if (st>384) dx=0;
            }
            dy=0;
            switch(level) {
                case 2: if (!(st%32))   dy=vs; break;
                case 4: if (!(st%8))    dy=vs; break;
                case 6: if (!(st%2))    dy=vs;
            }
            kmy+=dy;
            if (kmy>=152) vs=-1;
            if (kmy<=0) vs=1;
        break;
    }

    for (i=0;i<MAXENEMY;i++) {
        foes[i].x+=dx*foes[i].dx;
        foes[i].y+=dy;
        if ((foes[i].x >= (CX1-BRD)) && (foes[i].x < CX2) &&
            (foes[i].y >= (CY1-BRD)) && (foes[i].y < CY2) &&
            (foes[i].st==2))
                                foes[i].st=1;
        if (foes[i].st==1 || !(level%2)) {
            if (foes[i].y <  CY1-BRD)   foes[i].y += (CY2-CY1+BRD);
            if (foes[i].y >= CY2) {
                foes[i].y -= (CY2-CY1+BRD);
                foes[i].x += (CX2-CX1)/2;
            }
        }
        if (foes[i].st==1 || (level%2)) {
            if (foes[i].x <  CX1-BRD)   foes[i].x += (CX2-CX1+BRD);
            if (foes[i].x >= CX2)       foes[i].x -= (CX2-CX1+BRD);
        }
    }
    st=(st+1)%512;
    return(0);
}






